'use strict';

const reqPromise = require('request-promise');
const util = require('./util');
const config = require('./config.json');

function fetch(gitInfo) {
    const options = getRequestOptions(gitInfo);
    const resolveFetchBound = resolveFetch.bind(null, gitInfo.origPath);
    return reqPromise.get(options).then(
        resolveFetchBound
    ).catch(error => {
        throw error;
    });
}

function resolveFetch(origPath, result) {
    result = util.stringToJson(result);
    result.__mockfromgit__origPath = origPath;
    return result;
}

function getRequestOptions(gitInfo) {
    if (gitInfo.domain.indexOf('github') !== -1) {
        const urlSegments = [
            config.github.apiUrl,
            gitInfo.owner,
            gitInfo.repo,
            'contents',
            gitInfo.path
        ];
        let url = urlSegments.join('/');
        url += gitInfo.branch ? '?ref=' + gitInfo.branch : '';
        const token = gitInfo.token ? 'token ' +  gitInfo.token : '';

        const options = {
            url: url,
            headers: {
                'User-Agent': 'request',
                'Authorization': token
            }
        };
        return options;
    }
}

function fetchJson(gitInfo, requiredJsonPaths) {
    const jsonResponses = [];
    const fullJsonPaths = util.buildFullJsonPaths(requiredJsonPaths, gitInfo.path);

    fullJsonPaths.forEach(path => {
        gitInfo.path = path.fullPath;
        gitInfo.origPath = path.origPath;
        const resp = fetch(gitInfo);
        jsonResponses.push(resp);
    });

    return Promise.all(jsonResponses).then(res => {
        const jsonMap = {};
        res.forEach(result => {
            const str = util.base64ToString(result.content);
            const json = util.stringToJson(str);
            jsonMap[result.__mockfromgit__origPath] = json;
        });
        return jsonMap;
    });
}

module.exports = {
    fetch: fetch,
    fetchJson: fetchJson
};