'use strict';

const gitRepo = require('./git-repo');
const util = require('./util');
const config = require('./config.json');
const Module = module.constructor;

async function fetch(req, config) {
    if (!req) {
        throw new Error('Missing request object');
    }
    const token = config && config.token ? config.token : '';
    const gitInfo = {
        domain: req.params.domain,
        owner: req.params.owner,
        repo: req.params.repo,
        branch: req.params.branch,
        path: req.params['0'], /* path does not begin with slash */
        token: token
    };

    const result = await gitRepo.fetch(gitInfo);
    const contentStr = util.base64ToString(result.content);

    if (util.endsWith(result.name, '.json')) {
        return resolveJson(contentStr);
    } else if (util.endsWith(result.name, '.js')) {
        return resolveJs(req, contentStr, gitInfo);
    } else {
        throw new Error('Could not find json or js files.')
    }
}

async function resolveJs(req, contentStr, gitInfo) {
    const requiredJsonPaths = util.findRequiredJsonPaths(contentStr);

    if (requiredJsonPaths.length) {
        const jsonMap = await gitRepo.fetchJson(gitInfo, requiredJsonPaths);
        const originalRequire = Module.prototype.require;

        Module.prototype.require = function(path){
            if (jsonMap[path]) {
                return jsonMap[path];
            }
            return originalRequire.apply(this, arguments);
        };
        return executeJs(req, contentStr);
        
    } else {
        return executeJs(req, contentStr);
    }
};

function resolveJson(contentStr) {
    const json = util.stringToJson(contentStr);
    return json;
};

function executeJs(req, contentStr) {
    const jsFn = util.stringToFunction(contentStr);
    const fnResults = jsFn(req);
    return fnResults;
}

module.exports = {
    fetch: fetch,
    githubPath: config.github.path
};
