'use strict';

const esprima = require('esprima');
const _ = require('lodash');

function stringToFunction(str) {
    const Module = module.constructor;
    const m = new Module();
    m._compile(str, "");
    return m.exports;
}

function stringToJson(str) {
    try {
        return JSON.parse(str);
    } catch (error) {
        console.log(error);
    }
}

function base64ToString(base64Str) {
    const buff = Buffer.from(base64Str, 'base64');  
    return buff.toString('utf-8');
}

function endsWith(name, ending) {
    if (name) {
        return !!name.toLowerCase().endsWith(ending);
    }
    return false;
}

function findRequiredJsonPaths(str) {
    const paths = [];
    const parsed = esprima.parseModule(str);
    const expressions = [];

    gatherCallExpressions(parsed, expressions);

    for (let i = 0, n = expressions.length; i < n; i++) {
        const exp = expressions[i];
        const name = _.get(exp, 'callee.name');
        if (name === 'require') {
            const path = _.get(exp, 'arguments[0].value');
            if (path && path.search(/\.json$/i) !== -1) {
                paths.push(path);
            }
        }
    }
    return paths;
}

function gatherCallExpressions(parsed, ary) {
    if (Array.isArray(parsed)) {
        parsed.forEach(obj => {
            if (typeof obj === 'object' || Array.isArray(obj)) {
                gatherCallExpressions(obj, ary);
            }
        });
    } else if (parsed && typeof parsed === 'object') {
        if (parsed.type === 'CallExpression') {
            ary.push(parsed);
        } else {
            for (let key in parsed) {
                gatherCallExpressions(parsed[key], ary);
            }
        }
    }
}

/*
jsonPaths = [
    '../common/population.json',
    './location/cities.json',
    './contacts.json'
];
targetPath = 'mock/info/index.js';
*/
function buildFullJsonPaths(jsonPaths, targetPath) {
    const targetSegments = targetPath.split('/');
    targetSegments.pop(); // Remove the target file
    const requiredPaths = [];

    for (let i = 0, n = jsonPaths.length; i < n; i++) {
        const origPath = jsonPaths[i];
        const pathSegments = origPath.split('/');
        let targetSegmentsCopy = targetSegments.slice(0);

        if (pathSegments[0] === '.') {
            pathSegments.shift();
        } else {
            pathSegments.forEach(fp => {
                if (fp === '..') {
                    targetSegmentsCopy.pop();
                    pathSegments.shift();
                }
            });
        }

        const segments = targetSegmentsCopy.concat(pathSegments);
        const jsonPath = {
            origPath: origPath,
            fullPath: segments.join('/')
        };
        requiredPaths.push(jsonPath);
    }

    return requiredPaths;
}

module.exports = {
    stringToFunction: stringToFunction,
    stringToJson: stringToJson,
    base64ToString: base64ToString,
    endsWith: endsWith,
    findRequiredJsonPaths: findRequiredJsonPaths,
    buildFullJsonPaths: buildFullJsonPaths
};